import pyodbc
import pandas

class ODBCInterface:
    def __init__(self, server, username, password, dbname):
        self.server = server
        self.username = username
        self.password = password
        self.dbname = dbname
        
    def __connect(self): #private
        try:
            self.cnx = pyodbc.connect("""DRIVER={ODBC Driver 17 for SQL Server};
                                      SERVER=""" + str(self.server) + """;
                                      DATABASE=""" + str(self.dbname) + """;
                                      UID=""" + str(self.username) + """;
                                      PWD=""" + str(self.password))
            return True
        except pyodbc.Error as err:
            print(err)
            return False
            
    def select(self, query):
        if(not self.__connect()):
            return None
        try:
            cursor = self.cnx.cursor()
            cursor.execute(query) 
            output = []
            for row in cursor:
                inner_list = []
                for val in row:
                    inner_list.append(str(val).strip())  #remove trailing white spaces to create a dataframe
                output.append(inner_list)
            cursor.close()
            self.cnx.close()
            return pandas.DataFrame(output)
        except:
            print("Cannot perform the SELECT query.")
            return False



import seaborn as sns
import matplotlib.pyplot as plt

def dfSummary(data):
    try:
        return data.describe()  #statistical summary
    except:
        print("Unable to provide a statistical summary")
        return False
        
def colBoxPlot(data):
    boxplot_inputs = []
    for col in range(0, len(data.columns)):
        try:
            floatdata = data[col].astype(float) #converting each column to float
            boxplot_inputs.append((col, floatdata))
        except ValueError as err:
            print(f"Unable to create box plot for column [{col}]. Error: [{err}]")
            
    plot_flag = False
    for data in boxplot_inputs:
        box_plot = sns.boxplot(data[1], orient = "v", linewidth = 2.5)
        box_plot.set(xlabel = f"Column [{data[0]}]", ylabel = "Values")
        plt.show()
        plot_flag = True
    return plot_flag



import mysql.connector
import pandas as pd

class MySQLInterface:    
    def __init__(self, server, username, password, dbname):
        self.server = server
        self.username = username
        self.password = password
        self.dbname = dbname
        
    def __connect(self):        
        try:
            self.cnx = mysql.connector.connect(user=self.username, password=self.password, host=self.server, database=self.dbname)
            return True
        except mysql.connector.Error as err:
            print(err)
            return False
            
    def select(self, query):
        if(not self.__connect()):
            return None
        try:
            output = []
            cursor = self.cnx.cursor()
            cursor.execute(query)
            for row in cursor:
                inner_list = []
                for val in row:
                    inner_list.append(str(val).strip())
                output.append(inner_list)
            cursor.close()
            self.cnx.close()
            return pd.DataFrame(output)
        except:
            print("Cannot perform the SELECT query.")
            return False



try:
    from mysql_interface import MySQLInterface
    from odbc_interface import ODBCInterface
except ImportError:
    from .mysql_interface import MySQLInterface
    from .odbc_interface import ODBCInterface

# Generic class that calls the appropriate DB class based on user's input
class DBInterface(ODBCInterface, MySQLInterface):
    dbtypelist = ['sql_server', 'mysql']
    
    def __init__(self, dbtype, server, username, password, dbname):
        if dbtype not in DBInterface.dbtypelist:
            raise Exception("DB type not supported. Ensure it's one of the following - {}".format(DBInterface.dbtypelist))
            
        self.dbtype = dbtype
        # for MySQL
        if self.dbtype == DBInterface.dbtypelist[0]:
            ODBCInterface.__init__(self, server, username, password, dbname)
        # for SQL Server
        elif self.dbtype == DBInterface.dbtypelist[1]:
            MySQLInterface.__init__(self, server, username, password, dbname)
            
    def querySelect(self, query):
        # for MySQL
        if self.dbtype == DBInterface.dbtypelist[0]:
            return ODBCInterface.select(self, query)
        # for SQL Server
        elif self.dbtype == DBInterface.dbtypelist[1]:
            return MySQLInterface.select(self, query)



import numpy as np
import pandas as pd
import missingno

def show_nulls(input_df):
    if(not isinstance(input_df, pd.DataFrame)):
        print("Can't cleanup an object that is not a dataframe")
        return None
    
    input_df.replace("None", np.nan, inplace = True)
    
    # Matrix that displays data sparsity to see missing/Null values    
    missingno.matrix(input_df,inline=True)
    return missingno.matrix(input_df,inline=False)



from setuptools import setup, find_packages

with open('README.md') as infile:
	readme = infile.read()

setup(
    name='xuebadb',
    version='1.3',
    python_requires='>=3',
    packages= find_packages(exclude=['tests*']),
    install_requires=['mysql-connector-python', 'pandas', 'pyodbc', 'numpy', 'missingno', 'seaborn', 'matplotlib'],
    license='MIT',
    description='xuebadb is a python package that allows users to conveniently connect to and query from various database systems using a unified API. Currently, it supports connecting to MySQL and SQL Server database systems. The queries return Pandas dataframes which can then be cleaned up and analysed using one of the modules in the package.',
    long_description=readme,
    long_description_content_type='text/markdown',
    url ='https://github.com/vaghulb1992/xuebadb',
    author='Vaghul Aditya Balaji',
    author_email ='vaghulb1992@gmail.com' 
)